from volworld_common.api.CA import CA
from volworld_common.test.behave.BehaveUtil import BehaveUtil

from volworld_aws_api_common.test.behave.row_utils import w__get_row_items
from volworld_aws_api_common.test.behave.selenium_utils import w__click_element_by_dom_id, \
    w__get_element_by_shown_dom_id, w__get_element_by_presence_dom_id, w__assert_element_not_existing, \
    get_element_by_dom_id


def click_on_page_in_page_container(c, page_str: str):
    page_str = BehaveUtil.clear_string(page_str)
    w__click_element_by_dom_id(c, [CA.Page, page_str, CA.Button])


def assert_pagination_of_list_page_count(c, count: int, total_page_count: int, total_item_count_str: str = ''):
    rows = w__get_row_items(c)
    assert len(rows) == count, f"found [{len(rows)}] books != [{count}] = expect"

    pg1_id = [CA.Page, '1', CA.Button]
    # assert page button
    if total_page_count > 1:
        last_pg_id = [CA.Page, f"{total_page_count}", CA.Button]
        no_pg_id = [CA.Page, f"{total_page_count + 1}", CA.Button]
        print(f'last_pg_id = [{last_pg_id}]')
        print(f'no_pg_id = [{no_pg_id}]')
        w__get_element_by_presence_dom_id(c, last_pg_id)
        w__assert_element_not_existing(c, no_pg_id)

    # assert book count
    elm = get_element_by_dom_id(c, [CA.Page, CA.TotalCount])
    all_book_count = int(elm.get_attribute('innerHTML').strip())
    if len(total_item_count_str) > 0:
        total_book_count = BehaveUtil.clear_int(total_item_count_str)
        assert all_book_count == total_book_count

    # not showing page button
    if total_page_count <= 1:
        w__assert_element_not_existing(c, pg1_id)
        return

    # if total_page_count <= 1:
    #     w__assert_element_not_existing(c, [A.Page, A.Info])
    #     return
    # elm = w__get_element_by_shown_dom_id(c, [A.Page, A.Info])
    # info = get_elm_text(elm)
    # all_page_count = int(info.split('/')[1].strip())
    # assert total_page_count == all_page_count


def assert_current_page(c, page_str: str):
    elm = w__get_element_by_shown_dom_id(c, [CA.Page, BehaveUtil.clear_string(page_str), CA.Button])
    class_list = elm.get_attribute("class").split()
    found_curr_class = False
    for c in class_list:
        if c.find('StPagination_CurrButton') > -1:
            found_curr_class = True
            break
    assert found_curr_class


import json
import yaml
import os.path
from jsonschema.validators import RefResolver
from openapi_schema_validator import validate
from volworld_common.util.json import convert_json_by_attributes

from volworld_aws_api_common.test.api.OAPI import OAPI


class OpenApiValidation:
    YamlNamePath = None
    ApiYaml = None
    ApiComponent = None
    ApiPath = None

    def __init__(self):
        if self.ApiYaml is None:
            # curr_path = pathlib.Path(__file__).parent.resolve()
            # api_yaml_path = os.path.join(curr_path, f"../../api/doc/{OpenApiValidation.YamlName}")
            print(f"Load API Yaml from [{self.YamlNamePath}]")
            assert os.path.exists(self.YamlNamePath)
            with open(self.YamlNamePath, 'r') as stream:
                self.ApiYaml = yaml.load(stream, Loader=yaml.CLoader)
            assert self.ApiYaml is not None
            self.ApiComponent = RefResolver.from_schema({
                OAPI.Components: self.ApiYaml[OAPI.Components]
            })
            self.ApiPath = self.ApiYaml[OAPI.Paths]

    def validate_POST_request(self, req, path: str, attList):
        self.validate_request(req, path, OAPI.Post, attList)

    def validate_GET_request(self, req, path: str, attList):
        self.validate_request(req, path, OAPI.Get, attList)

    def validate_request(self, req, path: str, method: str, attList):
        req_fix = convert_json_by_attributes(req, attList)
        req_fix_json = json.loads(req_fix)
        print("path = ", path)
        print("method = ", method)

        if OAPI.RequestBody not in self.ApiPath[path][method]:
            return

        request_body = self.ApiPath[path][method][OAPI.RequestBody]
        assert request_body[OAPI.Required] is True,\
            f"RequestBody of path [{path}][{method}] is NOT required.\n{request_body}."
        req_schema = request_body[OAPI.Content][OAPI.ApplicationJson][OAPI.Schema]

        # print("req_fix_json = ", req_fix_json)
        # print("req_schema = ", req_schema)
        # print("tar = ", req_fix_json)
        # pprint(self.ApiComponent)

        validate(req_fix_json, req_schema, resolver=self.ApiComponent)

    def validate_POST_response(self, req, path: str, code, attList):
        self.validate_response(req, path, OAPI.Post, code, attList)

    def validate_GET_response(self, req, path: str, code, attList):
        self.validate_response(req, path, OAPI.Get, code, attList)

    def validate_response(self, res, path: str, method: str, code, attList):
        res_fix = convert_json_by_attributes(res, attList)
        res_fix_json = json.loads(res_fix)

        response_body = self.ApiPath[path][method][OAPI.Responses][str(code)]
        if '$ref' in response_body.keys():
            ref_val = response_body['$ref']
            if ref_val.startswith('#/paths/'):
                ref = ref_val.split('#/paths/')[1]
                ref_path = ref.split('/')[0].replace('~1', '/')
                ref_method = ref.split('/')[1]
                assert ref.split('/')[2] == 'responses'
                ref_code = ref.split('/')[3]
                response_body = self.ApiPath[ref_path][ref_method][OAPI.Responses][str(ref_code)]

        res_schema = response_body[OAPI.Content][OAPI.ApplicationJson][OAPI.Schema]

        # print("ApiComponent = ", self.ApiComponent)
        print("res_schema = ", res_schema)
        print("res_fix_json = ", res_fix_json)
        # pprint(self.ApiComponent)

        validate(res_fix_json, res_schema, resolver=self.ApiComponent)



from setuptools import setup, find_packages


setup(
    name='volworld_aws_api_common',
    version='0.1.138',
    license='MIT',
    author="Che-Wri, Chang",
    author_email='gobidesert.mf@gmail.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='',
    keywords='volworld',
    install_requires=[
          'aenum',
          'volworld_common',
      ],

)



from typing import Final
from volworld_common.api.CA import CA


# ====== A: Attribute ======
class AA(CA):

    CheckBox: Final[str] = "chkbx"

    Dialog: Final[str] = "dlg"

    Token: Final[str] = "tk"


AAList = [AA, CA]
